@@ -1,5 +1,5 @@
 /*-
- * Copyright (c) 2011-2015 Ganael LAPLANCHE <ganael.laplanche@martymac.org>
+ * Copyright (c) 2011-2020 Ganael LAPLANCHE <ganael.laplanche@martymac.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -168,7 +168,7 @@ fpart_hook(const char *cmd, const struct program_options *options,
         assert(live_num_files != NULL);
 
         if(options->verbose >= OPT_VERBOSE)
-            fprintf(stderr, "Executing pre-part #%d hook: '%s'\n",
+            fprintf(stderr, "Executing pre-part #%ju hook: '%s'\n",
                 *live_partition_index, cmd);
 
         /* FPART_HOOKTYPE (pre-part) */
@@ -191,7 +191,7 @@ fpart_hook(const char *cmd, const struct program_options *options,
         assert(live_num_files != NULL);
 
         if(options->verbose >= OPT_VERBOSE)
-            fprintf(stderr, "Executing post-part #%d hook: '%s'\n",
+            fprintf(stderr, "Executing post-part #%ju hook: '%s'\n",
                 *live_partition_index, cmd);
 
         /* FPART_HOOKTYPE (post-part) */
@@ -233,7 +233,7 @@ fpart_hook(const char *cmd, const struct program_options *options,
             retval = 1;
             goto cleanup;
         )
-        snprintf(env_fpart_partnumber_string, malloc_size, "%s=%d",
+        snprintf(env_fpart_partnumber_string, malloc_size, "%s=%ju",
             env_fpart_partnumber_name, *live_partition_index);
         if(push_env(env_fpart_partnumber_string, &envp) != 0) {
             retval = 1;
@@ -249,7 +249,7 @@ fpart_hook(const char *cmd, const struct program_options *options,
             retval = 1;
             goto cleanup;
         )
-        snprintf(env_fpart_partsize_string, malloc_size, "%s=%lld",
+        snprintf(env_fpart_partsize_string, malloc_size, "%s=%ju",
             env_fpart_partsize_name, *live_partition_size);
         if(push_env(env_fpart_partsize_string, &envp) != 0) {
             retval = 1;
@@ -265,7 +265,7 @@ fpart_hook(const char *cmd, const struct program_options *options,
             retval = 1;
             goto cleanup;
         )
-        snprintf(env_fpart_partnumfiles_string, malloc_size, "%s=%llu",
+        snprintf(env_fpart_partnumfiles_string, malloc_size, "%s=%ju",
             env_fpart_partnumfiles_name, *live_num_files);
         if(push_env(env_fpart_partnumfiles_string, &envp) != 0) {
             retval = 1;
@@ -378,21 +378,23 @@ handle_file_entry(struct file_entry **head, char *path, fsize_t size,
     assert(options != NULL);
 
     if(options->live_mode == OPT_LIVEMODE)
-        return (live_print_file_entry(path, size, options->out_filename,
-            options));
+        return (live_print_file_entry(path, size, options));
     else
         return (add_file_entry(head, path, size, options));
 }
 
 /* Print a file entry */
 int
-live_print_file_entry(char *path, fsize_t size, char *out_template,
+live_print_file_entry(char *path, fsize_t size,
     struct program_options *options)
 {
     assert(path != NULL);
     assert(options != NULL);
     assert(options->live_mode == OPT_LIVEMODE);
 
+    char *out_template = options->out_filename;
+    char *ln_term = (options->out_zero == OPT_OUT0) ? "\0" : "\n";
+
     /* beginning of a new partition */
     if(live_status.partition_num_files == 0) {
         /* very first pass of first partition, preload first partition */
@@ -406,7 +408,7 @@ live_print_file_entry(char *path, fsize_t size, char *out_template,
             if_not_malloc(live_status.filename, malloc_size,
                 return (1);
             )
-            snprintf(live_status.filename, malloc_size, "%s.%d", out_template,
+            snprintf(live_status.filename, malloc_size, "%s.%ju", out_template,
                 live_status.partition_index);
         }
 
@@ -438,14 +440,14 @@ live_print_file_entry(char *path, fsize_t size, char *out_template,
 
     if(out_template == NULL) {
         /* no template provided, just print to stdout */
-        fprintf(stdout, "%d (%lld): %s\n", live_status.partition_index, size,
+        fprintf(stdout, "%ju (%ju): %s\n", live_status.partition_index, size,
             path);
     }
     else {
         /* print to fd */
         size_t to_write = strlen(path);
         if((write(live_status.fd, path, to_write) != (ssize_t)to_write) ||
-            (write(live_status.fd, "\n", 1) != 1)) {
+            (write(live_status.fd, ln_term, 1) != 1)) {
             fprintf(stderr, "%s\n", strerror(errno));
             /* do not close(livefd) and free(live_status.filename) here because
                it will be useful and free'd in uninit_file_entries() below */
@@ -464,7 +466,7 @@ live_print_file_entry(char *path, fsize_t size, char *out_template,
             (live_status.partition_size >= options->max_size))) {
         /* display added partition */
         if(options->verbose >= OPT_VERBOSE)
-            fprintf(stderr, "Filled part #%d: size = %lld, %lld file(s)\n",
+            fprintf(stderr, "Filled part #%ju: size = %ju, %ju file(s)\n",
                 live_status.partition_index, live_status.partition_size,
                 live_status.partition_num_files);
 
@@ -557,7 +559,7 @@ add_file_entry(struct file_entry **head, char *path, fsize_t size,
 }
 
 /* Compare entries to list directories first
-   - compar() function used by fts_open() when in leaf dirs mode */
+   - compar() function used by fts_open() when in dirs_only or leaf_dirs mode */
 static int
 #if (defined(__linux__) || defined(__NetBSD__)) && !defined(EMBED_FTS)
 fts_dirsfirst(const FTSENT **a, const FTSENT **b)
@@ -613,16 +615,32 @@ init_file_entries(char *file_path, struct file_entry **head, fnum_t *count,
         FTS_XDEV : 0;
 
     char *fts_argv[] = { file_path, NULL };
-    if((ftsp = fts_open(fts_argv, fts_options,
-        (options->leaf_dirs == OPT_LEAFDIRS) ? &fts_dirsfirst : NULL)) == NULL) {
+
+    /* sort function */
+#if (defined(__linux__) || defined(__NetBSD__)) && !defined(EMBED_FTS)
+    int (*fts_sortfuncp)(const FTSENT **, const FTSENT **);
+#else
+    int (*fts_sortfuncp)(const FTSENT * const *, const FTSENT * const *);
+#endif
+    if((options->dirs_only == OPT_DIRSONLY) ||
+       (options->leaf_dirs == OPT_LEAFDIRS))
+        fts_sortfuncp = &fts_dirsfirst;
+    else
+        fts_sortfuncp = NULL;
+
+    if((ftsp = fts_open(fts_argv, fts_options, fts_sortfuncp)) == NULL) {
         fprintf(stderr, "%s: fts_open()\n", file_path);
         return (0);
     }
 
     /* current dir state */
-    unsigned char curdir_empty = 1;
-    unsigned char curdir_dirsfound = 0;
-    unsigned char curdir_addme = 0;
+    unsigned char file_as_argument = 1; /* by default, we assume file_path
+                                           points to a file, not a dir */
+    unsigned char curdir_empty = 1;     /* current dir is empty */
+    unsigned char curdir_dirsfound = 0; /* other dirs have been found in
+                                           current dir */
+    unsigned char curdir_addme = 0;     /* current dir must be added */
+    fsize_t curdir_size = 0;            /* current dir size */
 
     while((p = fts_read(ftsp)) != NULL) {
         switch (p->fts_info) {
@@ -638,9 +656,9 @@ init_file_entries(char *file_path, struct file_entry **head, fnum_t *count,
             {
                 fprintf(stderr, "%s: %s\n", p->fts_path,
                     strerror(p->fts_errno));
-                /* if requested by the -Z option,
+                /* if requested by the -zz option,
                    add directory anyway by simulating FTS_DP */
-                if(options->dnr_empty == OPT_DNREMPTY) {
+                if(options->dirs_include >= OPT_DNREMPTY) {
                     curdir_empty = 1;
                     goto add_directory;
                 }
@@ -665,24 +683,30 @@ init_file_entries(char *file_path, struct file_entry **head, fnum_t *count,
             case FTS_DP:
             {
 add_directory:
-                /* if empty_dirs display requested and current dir is empty,
-                   add directory entry */
-                if((options->empty_dirs == OPT_EMPTYDIRS) && curdir_empty)
+                /* if dirs_only mode activated or
+                   leaf_dirs mode activated and current directory is a leaf or
+                   at least empty dirs display requested and current dir is empty */
+                if((options->dirs_only == OPT_DIRSONLY) ||
+                    ((options->leaf_dirs == OPT_LEAFDIRS) && (!curdir_dirsfound)) ||
+                    ((options->dirs_include >= OPT_EMPTYDIRS) && curdir_empty))
                     curdir_addme = 1;
 
-                /* if leaf dirs mode activated and current directory is a leaf,
-                   add directory entry */
-                if((options->leaf_dirs == OPT_LEAFDIRS) && (!curdir_dirsfound))
+                /* if current directory has not been added by previous rules
+                   but we request all directory entries, we fake an empty dir
+                   to avoid a call to get_size() below as we want it with a
+                   size of 0 */
+                if((!curdir_addme) && (options->dirs_include >= OPT_ALLDIRS)) {
                     curdir_addme = 1;
+                    curdir_empty = 1;
+                }
 
                 /* add directory if necessary */
                 if(curdir_addme) {
-                    fsize_t curdir_size = 0;
                     char *curdir_entry_path = NULL;
 
                     /* check for name validity regarding include/exclude
-                       options; directory is a leaf */
-                    if(!valid_filename(p->fts_name, options, 1)) {
+                       options */
+                    if(!valid_file(p, options, 1)) {
                         if(options->verbose >= OPT_VERBOSE)
                             fprintf(stderr, "Skipping directory: '%s'\n",
                                 p->fts_path);
@@ -707,7 +731,7 @@ add_directory:
                         snprintf(curdir_entry_path, malloc_size, "%s",
                             p->fts_path);
 
-                    /* compute current dir size */
+                    /* adapt curdir_size for special cases */
                     if((p->fts_level > 0) &&
                         (options->cross_fs_boundaries == OPT_NOCROSSFSBOUNDARIES) &&
                         (p->fts_parent->fts_statp->st_dev != p->fts_statp->st_dev))
@@ -715,10 +739,20 @@ add_directory:
                            (non-root) directories */
                         curdir_size = 0;
                     else if(curdir_empty)
+                        /* we know that the current dir is empty (or that we
+                           fake an empty one), ensure curdir_size is 0 */
                         curdir_size = 0;
-                    else
+                    else if((options->dirs_only != OPT_DIRSONLY) &&
+                            ((options->leaf_dirs != OPT_LEAFDIRS) || (curdir_dirsfound)))
+                        /* when dirs_only mode activated or
+                           leaf_dirs mode activated and current directory is a
+                           leaf, then we can use curdir_size.
+                           In all other cases (e.g. when dir_depth requested and
+                           reached), we must compute the directory size
+                           recursively. */
                         curdir_size =
                             get_size(p->fts_accpath, p->fts_statp, options);
+                    /* else, trust curdir_size and leave it untouched */
 
                     /* add or display it */
                     if(handle_file_entry
@@ -741,16 +775,18 @@ reset_directory:
                 curdir_empty = 0;
                 curdir_dirsfound = 1;
                 curdir_addme = 0;
+                curdir_size = 0;
                 continue;
             }
 
             case FTS_D:
             {
+                file_as_argument = 0; /* argument was not a file */
                 curdir_empty = 1; /* enter directory, mark it as empty */
                 curdir_dirsfound = 0; /* no dirs found yet */
 
                 /* check for name validity regarding exclude options */
-                if(!valid_filename(p->fts_name, options, 0)) {
+                if(!valid_file(p, options, 0)) {
                     if(options->verbose >= OPT_VERBOSE)
                         fprintf(stderr, "Skipping directory: '%s'\n",
                             p->fts_path);
@@ -776,28 +812,38 @@ reset_directory:
             /* XXX default means remaining file types:
                FTS_F, FTS_SL, FTS_SLNONE, FTS_DEFAULT */
             {
-                curdir_empty = 0; /* mark current dir as non empty */
+                fsize_t curfile_size = 0;
 
-                /* check for name validity regarding include/exclude options,*/
-                if(!valid_filename(p->fts_name, options, 1)) {
+                /* check for name validity regarding include/exclude options */
+                if(!valid_file(p, options, 1)) {
                     if(options->verbose >= OPT_VERBOSE)
                         fprintf(stderr, "Skipping file: '%s'\n", p->fts_path);
                     continue;
                 }
 
-                /* skip file entry when in leaf dirs mode (option -D) and no
-                   directory has been found in current directory (i.e. we are
-                   in a leaf directory). We must have visited all directories
-                   first ; this is achieved by using a compar() function with
-                   fts_open() */
-                if((options->leaf_dirs == OPT_LEAFDIRS) && (!curdir_dirsfound))
+                /* get current file size and add it to our current directory
+                   size. We must have visited all directories first for that
+                   total to be right ; this is achieved by using a compar()
+                   function with fts_open() */
+                curfile_size = get_size(p->fts_accpath, p->fts_statp, options);
+
+                curdir_empty = 0; /* mark current dir as non empty */
+                curdir_size += curfile_size;
+
+                /* skip file entry when in dirs_only mode (option -E) or
+                   in leaf_dirs mode (option -D) and no directory has been
+                   found in current directory (i.e. we are in a leaf directory).
+                   We must have visited all directories first and file entry must
+                   not explicitly point to a file (in such a case, file entry has
+                   been explicitly given as argument so we must add it) */
+                if((!file_as_argument) &&
+                    ((options->dirs_only == OPT_DIRSONLY) ||
+                    ((options->leaf_dirs == OPT_LEAFDIRS) && (!curdir_dirsfound))))
                     continue;
 
                 /* add or display it */
                 if(handle_file_entry
-                    (head, p->fts_path,
-                    get_size(p->fts_accpath, p->fts_statp, options),
-                    options) == 0)
+                    (head, p->fts_path, curfile_size, options) == 0)
                     (*count)++;
                 else {
                     fprintf(stderr, "%s(): cannot add file entry\n", __func__);
@@ -847,7 +893,7 @@ uninit_file_entries(struct file_entry *head, struct program_options *options)
         /* display added partition */
         if((options->verbose >= OPT_VERBOSE) &&
             (live_status.partition_num_files > 0))
-            fprintf(stderr, "Filled part #%d: size = %lld, %lld file(s)\n",
+            fprintf(stderr, "Filled part #%ju: size = %ju, %ju file(s)\n",
                 live_status.partition_index, live_status.partition_size,
                 live_status.partition_num_files);
 
@@ -883,16 +929,20 @@ uninit_file_entries(struct file_entry *head, struct program_options *options)
 /* Print a double-linked list of file_entries from head
    - if no filename template given, print to stdout */
 int
-print_file_entries(struct file_entry *head, char *out_template,
-    pnum_t num_parts)
+print_file_entries(struct file_entry *head, pnum_t num_parts,
+    struct program_options *options)
 {
     assert(head != NULL);
     assert(num_parts > 0);
+    assert(options != NULL);
+
+    char *out_template = options->out_filename;
+    char *ln_term = (options->out_zero == OPT_OUT0) ? "\0" : "\n";
 
     /* no template provided, just print to stdout and return */
     if(out_template == NULL) {
         while(head != NULL) {
-            fprintf(stdout, "%d (%lld): %s\n", head->partition_index,
+            fprintf(stdout, "%ju (%ju): %s\n", head->partition_index,
                 head->size, head->path);
             head = head->nextp;
         }
@@ -926,7 +976,7 @@ print_file_entries(struct file_entry *head, char *out_template,
                      close(fd[i]);
                 return (1);
             )
-            snprintf(out_filename, malloc_size, "%s.%d", out_template,
+            snprintf(out_filename, malloc_size, "%s.%ju", out_template,
                 (current_chunk * PRINT_FE_CHUNKS) + current_file_entry);
 
             if((fd[current_file_entry] =
@@ -948,7 +998,7 @@ print_file_entries(struct file_entry *head, char *out_template,
                (head->partition_index < ((current_chunk + 1) * PRINT_FE_CHUNKS))) {
                 size_t to_write = strlen(head->path);
                 if((write(fd[head->partition_index % PRINT_FE_CHUNKS], head->path, to_write) != (ssize_t)to_write) ||
-                    (write(fd[head->partition_index % PRINT_FE_CHUNKS], "\n", 1) != 1)) {
+                    (write(fd[head->partition_index % PRINT_FE_CHUNKS], ln_term, 1) != 1)) {
                     fprintf(stderr, "%s\n", strerror(errno));
                     /* close all open descriptors */
                     pnum_t i;
