@@ -1,5 +1,5 @@
 /*-
- * Copyright (c) 2011-2015 Ganael LAPLANCHE <ganael.laplanche@martymac.org>
+ * Copyright (c) 2011-2020 Ganael LAPLANCHE <ganael.laplanche@martymac.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -47,6 +47,8 @@ init_options(struct program_options *options)
     assert(DFLT_OPT_MAX_SIZE >= 0);
     assert((DFLT_OPT_ARBITRARYVALUES == OPT_NOARBITRARYVALUES) ||
            (DFLT_OPT_ARBITRARYVALUES == OPT_ARBITRARYVALUES));
+    assert((DFLT_OPT_OUT0 == OPT_NOOUT0) ||
+           (DFLT_OPT_OUT0 == OPT_OUT0));
     assert((DFLT_OPT_ADDSLASH == OPT_NOADDSLASH) ||
            (DFLT_OPT_ADDSLASH == OPT_ADDSLASH));
     assert((DFLT_OPT_VERBOSE == OPT_NOVERBOSE) ||
@@ -56,13 +58,15 @@ init_options(struct program_options *options)
            (DFLT_OPT_FOLLOWSYMLINKS == OPT_NOFOLLOWSYMLINKS));
     assert((DFLT_OPT_CROSSFSBOUNDARIES == OPT_NOCROSSFSBOUNDARIES) ||
            (DFLT_OPT_CROSSFSBOUNDARIES == OPT_CROSSFSBOUNDARIES));
-    assert((DFLT_OPT_EMPTYDIRS == OPT_NOEMPTYDIRS) ||
-           (DFLT_OPT_EMPTYDIRS == OPT_EMPTYDIRS));
-    assert((DFLT_OPT_DNREMPTY == OPT_NODNREMPTY) ||
-           (DFLT_OPT_DNREMPTY == OPT_DNREMPTY));
+    assert((DFLT_OPT_DIRSINCLUDE == OPT_NOEMPTYDIRS) ||
+           (DFLT_OPT_DIRSINCLUDE == OPT_EMPTYDIRS) ||
+           (DFLT_OPT_DIRSINCLUDE == OPT_DNREMPTY) ||
+           (DFLT_OPT_DIRSINCLUDE == OPT_ALLDIRS));
     assert(DFLT_OPT_DIR_DEPTH >= OPT_NODIRDEPTH);
     assert((DFLT_OPT_LEAFDIRS == OPT_NOLEAFDIRS) ||
            (DFLT_OPT_LEAFDIRS == OPT_LEAFDIRS));
+    assert((DFLT_OPT_DIRSONLY == OPT_NODIRSONLY) ||
+           (DFLT_OPT_DIRSONLY == OPT_DIRSONLY));
     assert((DFLT_OPT_LIVEMODE == OPT_NOLIVEMODE) ||
            (DFLT_OPT_LIVEMODE == OPT_LIVEMODE));
     assert(DFLT_OPT_PRELOAD_SIZE >= 0);
@@ -76,6 +80,7 @@ init_options(struct program_options *options)
     options->in_filename = NULL;
     options->arbitrary_values = DFLT_OPT_ARBITRARYVALUES;
     options->out_filename = NULL;
+    options->out_zero = DFLT_OPT_OUT0;
     options->add_slash = DFLT_OPT_ADDSLASH;
     options->verbose = DFLT_OPT_VERBOSE;
     options->follow_symbolic_links = DFLT_OPT_FOLLOWSYMLINKS;
@@ -88,10 +93,10 @@ init_options(struct program_options *options)
     options->nexclude_files = 0;
     options->exclude_files_ci = NULL;
     options->nexclude_files_ci = 0;
-    options->empty_dirs = DFLT_OPT_EMPTYDIRS;
-    options->dnr_empty = DFLT_OPT_DNREMPTY;
+    options->dirs_include = DFLT_OPT_DIRSINCLUDE;
     options->dir_depth = DFLT_OPT_DIR_DEPTH;
     options->leaf_dirs = DFLT_OPT_LEAFDIRS;
+    options->dirs_only = DFLT_OPT_DIRSONLY;
     options->live_mode = DFLT_OPT_LIVEMODE;
     options->pre_part_hook = NULL;
     options->post_part_hook = NULL;
@@ -112,10 +117,10 @@ uninit_options(struct program_options *options)
     if(options->pre_part_hook != NULL)
         free(options->pre_part_hook);
     options->live_mode = DFLT_OPT_LIVEMODE;
+    options->dirs_only = DFLT_OPT_DIRSONLY;
     options->leaf_dirs = DFLT_OPT_LEAFDIRS;
     options->dir_depth = DFLT_OPT_DIR_DEPTH;
-    options->dnr_empty = DFLT_OPT_DNREMPTY;
-    options->empty_dirs = DFLT_OPT_EMPTYDIRS;
+    options->dirs_include = DFLT_OPT_DIRSINCLUDE;
     if(options->exclude_files_ci != NULL)
         str_cleanup(&(options->exclude_files_ci),
             &(options->nexclude_files_ci));
@@ -132,6 +137,7 @@ uninit_options(struct program_options *options)
     options->follow_symbolic_links = DFLT_OPT_FOLLOWSYMLINKS;
     options->verbose = DFLT_OPT_VERBOSE;
     options->add_slash = DFLT_OPT_ADDSLASH;
+    options->out_zero = DFLT_OPT_OUT0;
     if(options->out_filename != NULL)
         free(options->out_filename);
     options->arbitrary_values = DFLT_OPT_ARBITRARYVALUES;
