@@ -1,5 +1,5 @@
 /*-
- * Copyright (c) 2011-2015 Ganael LAPLANCHE <ganael.laplanche@martymac.org>
+ * Copyright (c) 2011-2020 Ganael LAPLANCHE <ganael.laplanche@martymac.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -55,6 +55,11 @@ struct program_options {
     unsigned char arbitrary_values;
 /* output file (option -o); NULL = stdout, "filename" */
     char *out_filename;
+/* add a null character after filename in file lists */
+#define OPT_NOOUT0                  0
+#define OPT_OUT0                    1
+#define DFLT_OPT_OUT0               OPT_NOOUT0
+    unsigned char out_zero;
 /* add slash to directories (option -e) */
 #define OPT_NOADDSLASH              0
 #define OPT_ADDSLASH                1
@@ -88,25 +93,27 @@ struct program_options {
 /* exclude files, case insensitive (option -X) */
     char **exclude_files_ci;
     unsigned int nexclude_files_ci;
-/* include empty directories (option -z) */
+/* include certain directories (option -z) */
 #define OPT_NOEMPTYDIRS             0
-#define OPT_EMPTYDIRS               1
-#define DFLT_OPT_EMPTYDIRS          OPT_NOEMPTYDIRS
-    unsigned char empty_dirs;
-/* treat un-readable directories as empty (option -Z) */
-#define OPT_NODNREMPTY             0
-#define OPT_DNREMPTY               1
-#define DFLT_OPT_DNREMPTY          OPT_NODNREMPTY
-    unsigned char dnr_empty;
+#define OPT_EMPTYDIRS               1   /* include empty directories */
+#define OPT_DNREMPTY                2   /* also add un-readable directories */
+#define OPT_ALLDIRS                 3   /* include all directories */
+#define DFLT_OPT_DIRSINCLUDE        OPT_NOEMPTYDIRS
+    unsigned char dirs_include;
 /* display directories after certain depth (option -d) */
 #define OPT_NODIRDEPTH              -1
 #define DFLT_OPT_DIR_DEPTH          OPT_NODIRDEPTH
     int dir_depth;
 /* pack leaf directories (option -D) */
-#define OPT_NOLEAFDIRS               0
-#define OPT_LEAFDIRS                 1
-#define DFLT_OPT_LEAFDIRS            OPT_NOLEAFDIRS
+#define OPT_NOLEAFDIRS              0
+#define OPT_LEAFDIRS                1
+#define DFLT_OPT_LEAFDIRS           OPT_NOLEAFDIRS
     unsigned char leaf_dirs;
+/* pack directories only (option -E) */
+#define OPT_NODIRSONLY              0
+#define OPT_DIRSONLY                1
+#define DFLT_OPT_DIRSONLY           OPT_NODIRSONLY
+    unsigned char dirs_only;
 /* live mode (option -L) */
 #define OPT_NOLIVEMODE              0
 #define OPT_LIVEMODE                1
