@@ -1,5 +1,5 @@
 /*-
- * Copyright (c) 2011-2015 Ganael LAPLANCHE <ganael.laplanche@martymac.org>
+ * Copyright (c) 2011-2020 Ganael LAPLANCHE <ganael.laplanche@martymac.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -150,7 +150,7 @@ print_partitions(struct partition *head)
 {
     pnum_t i = 0;
     while(head != NULL) {
-        fprintf(stderr, "Part #%d: size = %lld, %lld file(s)\n", i,
+        fprintf(stderr, "Part #%ju: size = %ju, %ju file(s)\n", i,
             head->size, head->num_files);
         head = head->nextp;
         i++;
