@@ -1,5 +1,5 @@
 /*-
- * Copyright (c) 2011-2015 Ganael LAPLANCHE <ganael.laplanche@martymac.org>
+ * Copyright (c) 2011-2020 Ganael LAPLANCHE <ganael.laplanche@martymac.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -34,6 +34,13 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 
+/* fts(3) */
+#if defined(EMBED_FTS)
+#include "fts.h"
+#else
+#include <fts.h>
+#endif
+
 /* fnmatch(3) and FNM_CASEFOLD
    FNM_CASEFOLD is a GNU extension and may not be available */
 #include <fnmatch.h>
@@ -56,6 +63,7 @@
     { while((head) && (head)->nextp) { (head) = (head)->nextp; } }
 
 #define min(x, y) (((x) <= (y)) ? (x) : (y))
+#define max(x, y) (((x) >= (y)) ? (x) : (y))
 
 #define if_not_malloc(ptr, size, err_action)                            \
     ptr = malloc(size);                                                 \
@@ -71,15 +79,18 @@
         err_action                                                      \
     }
 
+uintmax_t char_to_multiplier(const char c);
 unsigned int get_num_digits(double i);
 fsize_t get_size(char *file_path, struct stat *file_stat,
     struct program_options *options);
 char *abs_path(const char *path);
 int str_push(char ***array, unsigned int *num, const char * const str);
 void str_cleanup(char ***array, unsigned int *num);
-int str_match(const char * const * const array, const unsigned int num,
-    const char * const str, const unsigned char ignore_case);
-int valid_filename(char *filename, struct program_options *options,
+int str_is_negative(const char * const str);
+uintmax_t str_to_uintmax(const char *str, const unsigned char handle_multiplier);
+int file_match(const char * const * const array, const unsigned int num,
+    const FTSENT * const p, const unsigned char ignore_case);
+int valid_file(const FTSENT * const p, struct program_options *options,
     unsigned char is_leaf);
 char ** clone_env(void);
 int push_env(char *str, char ***env);
